/* -------------------------------------------------------------------------- */

#include <stdio.h>
#include <string.h>

#include "ima.h"

/* -------------------------------------------------------------------------- */

typedef struct riff_head {
	unsigned int sign;
	unsigned int size;
	unsigned int type;
} riff_head;

/* -------------------------------------------------------------------------- */

typedef struct riff_chunk {
	unsigned int type;
	unsigned int size;
} riff_chunk;

/* -------------------------------------------------------------------------- */

typedef struct wave_format {
	unsigned short tag;
	unsigned short nch;
	unsigned int freq;
	unsigned int rate;
	unsigned short align;
	unsigned short bits;
} wave_format;

/* -------------------------------------------------------------------------- */

int sr;

void encode(char *inname, char *outname)
{
	FILE *in, *out;
	riff_head head;
	riff_chunk chunk;
	wave_format format;
	size_t remain, step;
	char buf[256], ima[64];
	ima_state state[2];

	memset(&format, 0, sizeof(format));

	if(!(in = fopen(inname, "rb")))
	{
		printf("%s: can't open\n", inname);
	}
	else
	{
		if(!(out = fopen(outname, "wb")))
		{
			printf("%s: can't create\n", outname);
		}
		else
		{
			if( ! ((fread(&head, sizeof(head), 1, in)) &&
				(head.sign == 'FFIR') &&
				(head.type == 'EVAW')) )
			{
				printf("%s: not waveform file\n", inname);
			}
			else
			{
				while(fread(&chunk, sizeof(chunk), 1, in))
				{
					remain = chunk.size;

					if(chunk.type == ' tmf')
					{
						if(remain >= sizeof(format))
							remain -= fread(&format, 1, sizeof(format), in);
					}
					
					else if(chunk.type == 'atad')
					{
						if( !  ((format.tag == 1) && 
							//(format.freq == 24000) &&
							(format.bits == 16) &&
							(format.nch == 1) ))
						{
							printf("%s: unsupported format. "
								"required: PCM/16 bit signed LE/mono\n", inname);
						}
						else
						{
                                                        sr=format.freq;

							puts("encoding stream...");

							memset(&state, 0, sizeof(state));
							
							step = remain;
							while(remain)
							{
								if(step > sizeof(buf))
									step = sizeof(buf);

								if(!(step = fread(buf, 1, step, in)))
									break;

								ima_encode_mono /*stereo*/ (state, ima, buf, step);

								fwrite(ima, 1, step/4, out);

								remain -= step;
								step = sizeof(buf);
							}

							puts("finished encoding");
						}
					}

					fseek(in, (long)remain, SEEK_CUR);
				}
			}
			fclose(out);
		}
		fclose(in);
	}

	puts("");
}

/* -------------------------------------------------------------------------- */

void decode(char *inname, char *outname)
{
	FILE *in, *out;
	char buf[64], pcm[256];
	size_t step;
	ima_state state[2];
	riff_head head;
	riff_chunk chunk;
	wave_format format;
	size_t len;

	if(!(in = fopen(inname, "rb")))
	{
		printf("%s: can't open\n", inname);
	}
	else
	{
		if(!(out = fopen(outname, "wb")))
		{
			printf("%s: can't create\n", outname);
		}
		else
		{
			fseek(in, 0, SEEK_END);
			len = ftell(in);

			puts("writing wavefile header...");

			head.sign = 'FFIR';
			head.type = 'EVAW';
			head.size = sizeof(head) + 
				sizeof(chunk)*2 + 
				sizeof(format) + 
				len*4;
			fwrite(&head, sizeof(head), 1, out);

			chunk.type = ' tmf';
			chunk.size = sizeof(format);
			fwrite(&chunk, sizeof(chunk), 1, out);

			format.tag = 1;
			format.freq = sr; //24000
			format.bits = 16;
			format.nch = 1;
			format.align = format.bits * format.nch / 8;
			format.rate = format.freq * format.align;
			fwrite(&format, sizeof(format), 1, out);

			chunk.type = 'atad';
			chunk.size = (unsigned)len*4;
			fwrite(&chunk, sizeof(chunk), 1, out);

			puts("decoding stream...");
		
			memset(&state, 0, sizeof(state));

			rewind(in);
			while((step = fread(buf, 1, sizeof(buf), in)))
			{
				ima_decode_mono /*stereo*/ (state, pcm, buf, step);
				fwrite(pcm, 1, step*4, out);
			}

			puts("decoding finished");
			
			fclose(out);
		}
		fclose(in);
	}

	puts("");
}

/* -------------------------------------------------------------------------- */

int main(int argc, char **argv)
{
 encode(argv[1],argv[2]);
 decode(argv[2],argv[3]);
 return 0;
}

/* -------------------------------------------------------------------------- */

// by redsh
// mailto:californ251@gmail.com
